import SimpleHTTPServer
import SocketServer
import os
import time

class BasicServer(object):
    def __init__(self, path, reporter, port=None):
        PORT = 8000

        if port != None:
            PORT = port

        os.chdir(path) 

        try:
            reporter.log("Starting a server" , "blue")
            # Wait for the port to free up, dirty :)
            time.sleep(1)
            Handler = SimpleHTTPServer.SimpleHTTPRequestHandler
            httpd = SocketServer.TCPServer(("", PORT), Handler)
        except KeyboardInterrupt:
            reporter.log("Closing server" , "red")
            return
        
        try:
            reporter.log("Started server at 127.0.0.1:%d" % (PORT), "green")
            httpd.serve_forever()
        
        except KeyboardInterrupt: 
            httpd.server_close()
            reporter.log("Closed server at at 127.0.0.1:%d" % (PORT), "red")




import os
import sys
import argparse
import json
import yaml
import importlib
import inspect
from blinker import signal

import Reader as Reader
import JinjaStaticRenderer as renderer
import Core as Core
import Extensions as Extensions
import defaults as defaults
import NewProject as generate
import BasicServer as serve
import Watcher as watch
import messages as messages

class Wrangler(object):

    defaults = defaults.load_defaults()

    # Default config just loads the YAML config file
    config = {
        "wrangler": {
            "verbose": False,
            "force": False,
            "nocache": False,
            "input_dir": False,
            "output_dir": False,
            "data_formats": None
        },
        "site": {},
        "extensions": {}
    }

    # User config path 
    config_path = 'wrangler.yaml'
    extension_results = {}

    # Hmm it turns out there's not going to be any global options at all, the whole app
    # works best as a series of subcommands with very independent options.
    def parse_args(self, args=None):
        d = '\033[32mProcess recursive directories of JSON files through Jinja2 templates. \033[0m'
        parser = argparse.ArgumentParser(description=d)
        
        subparsers = parser.add_subparsers(dest="subparser", help='sub-command help')
        
        # ---------------------------------------------------------------------
        # wrangler create .
        # ---------------------------------------------------------------------
        # Creating a new project/scaffolding
         
        parser_generate = subparsers.add_parser('create', help='create help')
        parser_generate.add_argument('path', type=str, help='wrangler create path')
       

        # ---------------------------------------------------------------------
        # wrangler serve www
        # ---------------------------------------------------------------------
        # Serving up content on a specified URL
        
        parser_serve = subparsers.add_parser('serve', help='server help')
        parser_serve.add_argument('path', type=str, help='wrangler server path')
        parser_serve.add_argument("-p", "--port",
                            help='\033[34mThe port number, eg `8000`\033[0m', type=int)

        # ---------------------------------------------------------------------
        # wrangler build [i] [o]
        # ---------------------------------------------------------------------
        # Running the build command
        
        parser_build = subparsers.add_parser('build', help='build help')
        parser_build.add_argument('input_dir',
                            help='\033[34mInput directory such as `content`\033[0m')
        parser_build.add_argument('output_dir',
                            help='\033[34mOutput directory such as `www`\033[0m')
        parser_build.add_argument("-t", "--templates_dir",
                            help='\033[34mTemplate directory such as `templates`\033[0m')
        parser_build.add_argument("-c", "--config",
                            help='\033[34mPath to `wrangler.json`\033[0m')
        parser_build.add_argument("-o", "--output_file_extension",
                            help='\033[34mType of files to output such as `html`\033[0m')
        parser_build.add_argument("-d", "--data_format",
                            help='\033[34mType of files to match in input_dir, such as `json`\033[0m')
        parser_build.add_argument("-v", "--verbose", action="store_true",
                            help='\033[34mPrint all the plumbing\033[0m')
        parser_build.add_argument("-f", "--force", action="store_true",
                            help='\033[34mSmash out all the templates regardless of mtime\033[0m')
        parser_build.add_argument("-n", "--nocache", action="store_true",
                            help='\033[34mTurn off data persistence\033[0m')


        # ---------------------------------------------------------------------
        # wrangler watch [i] [o]
        # ---------------------------------------------------------------------
        # Same config as build..
        
        parser_watch = subparsers.add_parser('watch', help='build help')
        parser_watch.add_argument('input_dir',
                            help='\033[34mInput directory such as `content`\033[0m')
        parser_watch.add_argument('output_dir',
                            help='\033[34mOutput directory such as `www`\033[0m')
        parser_watch.add_argument("-f", "--force", action="store_true",
                            help='\033[34mSmash out all the templates regardless of mtime\033[0m')
        parser_watch.add_argument("-n", "--nocache", action="store_true",
                            help='\033[34mTurn off data persistence\033[0m')
        parser_watch.add_argument("-v", "--verbose", action="store_true",
                            help='\033[34mPrint all the plumbing\033[0m')
        
        parser_clean = subparsers.add_parser('clean', help='clean help')
        return parser.parse_args(args)


    def update_config(self, args):
        userConfig = None

        # Load the defaults
        self.config.update( self.defaults )

        # Check for custom config in the arguments
        if (hasattr(args, "config")):
            if args.config != None:
                self.config_path = args.config

        # Load custom config if it exists
        _path = u"%s" % (self.config_path)

        if os.path.exists( _path ):
            userConfig = yaml.load( file(_path) )
            self.config.update( userConfig )

        # # Apply command line flags if set, ignore Nones.
        self.config["wrangler"].update((k, v) for k, v in args.__dict__.iteritems() if v is not None)
        
        # If there's a library path, load all the modules
        if "lib_path" in self.config["wrangler"]:
            self.load_classes(self.config["wrangler"]["lib_path"])


    def main(self, args=None):
        args = self.parse_args(args)

        self.update_config(args)

        # Reporter only really needs to check the verbosity level
        self._reporter = Core.Reporter(self.config, self.config_path)

        if "subparser" in args:
            
            if args.subparser == "create":
                generate.NewProject(args.path, self._reporter)
                exit()
            
            if args.subparser == "serve":
                port = None
                if args.port:
                    port = args.port 
                serve.BasicServer(args.path, self._reporter, port)
            
            if args.subparser == "build":
                self._reporter.log("Wrangling %s" % (self.config['wrangler']['input_dir']),"green")
                self.render()

            if args.subparser == "watch":

                watch_init = signal('watch_init')
                watch_change = signal('watch_change')
                
                watch_init.connect(self.on_watch_ready)
                watch_change.connect(self.on_watch_change)
                watcher = watch.Watcher(self.config["wrangler"]["input_dir"], self.config["wrangler"]["templates_dir"])

            if args.subparser == "clean":
                # self.update_config(args)
                files = [self.config["wrangler"]["compiled_templates_file"], self.config["wrangler"]["build_cache_file"]]
                
                for f in files:
                    if os.path.exists(f):
                        try:
                            os.remove(f)
                            self._reporter.log("Cleaning up: %s" % (f), "blue")
                        except:
                            self._reporter.log("Couldn't access: %s" % (f), "red")
                self._reporter.log("Done", "green")
                exit()

        else:
            return None

    def on_watch_ready(self, sender):
        conf = self.config["wrangler"]
        self._reporter.log(messages.watch_start % (conf['input_dir'], conf['templates_dir']), "blue")

    def on_watch_change(self, sender):
        self._reporter.log(messages.watch_change % (sender.src_path), "green")
        self.render()

    def render(self):
        conf = self.config["wrangler"]
        self._reader = Reader.Reader(self.config, self._reporter)
        self._writer = Core.Writer(conf["output_dir"], conf["output_file_extension"], self._reporter)
        self._renderer = renderer.JinjaStaticRenderer(self.config, self._reporter, self._writer)
        self.graph = self._reader.fetch()

        total_nodes = 0

        for key, node in self.graph.all().items():
            if node.tag == 'file':
                cargo = node.get_cargo()     
                if cargo:
                    cargo.set_output_path(self._writer.generate_output_path(cargo.relpath()))
                total_nodes += 1

        

        before = signal('wranglerBeforeRender')
        result = before.send('wrangler',
                    config=self.config,
                    renderer=self._renderer,
                    reporter=self._reporter,
                    nodes=self.graph)

        self.process_extensions()
        rendered = 0
        

        for key, node in self.graph.all().items():
            if node.tag == 'file':
                # Leaky boat.
                cargo = node.get_cargo()
                if cargo:
                    cargo.set_parents(node.get_parents())
                    cargo.set_children(node.get_child_pages())
                    cargo.set_siblings(node.get_siblings())
                    cargo.set_parents_siblings(node.get_parents_siblings())
                    
                    _rel = cargo.get_related()

                    if _rel != None:
                        arr = self.graph.all()
                        related = []
                        for item in _rel:
                            _item = arr.get(item)
                            if _item:
                                related.append(_item)
                        
                        cargo.set_related_nodes(related)

                    # The important bit that does the saving and the rendering
                    if self._writer.save(
                        self._renderer.render(
                            cargo,
                            meta = cargo.get_metadata(),
                            data = cargo.get_content(),
                            site = self.config["site"],
                            extensions = self.extension_results
                            )
                        ):
                        rendered += 1
                    cargo.cleanup()
                    del cargo
    

        after = signal('wranglerAfterRender')
        final_result = after.send('wrangler',
                    config=self.config,
                    renderer=self._renderer,
                    reporter=self._reporter,
                    nodes=self.graph)
        
        if rendered > 0:
            self._reporter.log(messages.built_n_of_n % (rendered, total_nodes, conf["output_dir"]), "green")
            self._reporter.set_last_build_time()
        else:
             self._reporter.log(messages.nochange % (conf["input_dir"], conf["templates_dir"]), "red")


    def load_classes(self, views):
        """
        Pull in files from the lib directory
        """
        classfiles = [os.path.join(dirpath, f)
            for dirpath, dirnames, files in os.walk(views)
            for f in files if f.endswith('.py')]

        path = list(sys.path)
        sys.path.insert(0, views)

        moduleNames = []

        for f in classfiles:
            moduleNames.append(os.path.basename(f.replace(".py", "")))

        if len(moduleNames) > 0:
            try:
                map(__import__, moduleNames);
            finally:
                # restore the syspath
                sys.path[:] = path 


    def process_extensions(self):
        """
        Load extensions from the core and import any from the site/lib directory too
        """
        sig = signal("wranglerExtension")
        results = sig.send('wrangler')

        # Results returns a list of tuples.
        # First one will be the decorator, second will be the fn
        for result in results:
            responder = result[1]
            self.extension_results[responder.__name__] = responder(config=self.config, reporter=self._reporter, nodes=self.graph)

        # print self.extension_results

def start():
    sys.exit(Wrangler().main())

if __name__ == '__main__':
    start()



# examples.py
import wrangler.Core as Core
from wrangler.Core import extension, before_render, after_render, load_item, save_item, render_item, template_filter
from jinja2 import contextfilter


# Parser 

# class RestructuredTextParser(Core.Parser):
#     accepts = ["rst"]

#     def interpret(self, file_contents):
#         return rst.load(file_contents)


# Page subclass (assumes you've set a class: MyPage in your page meta)

# class MyPage(Page):
#     def get_content(self):
#         return self.data["data"]
#     def get_metadata(self):
#         return self.data["meta"]



# Hooks

# @before_render
# def before(**kw):
#     nodes = kw['nodes']
#     config = kw['config']
#     renderer = kw['renderer']
#     reporter = kw['reporter']
#     print "Hey, I'm a hook!"
#     return "foo!"

# @after_render
# def after(**kw):
#     nodes = kw['nodes']
#     config = kw['config']
#     renderer = kw['renderer']
#     reporter = kw['reporter']
#     print "Hey, I'm a hook!"
#     return ""


# Extension

# @extension
# def my_extension(sender, **kwargs):
#     # Add some config to your wrangler.yaml file and access it here: 
#     config = kwargs['config']['extensions']['my_extension']
#     return "This is my basic extension!"



# Filters

# @template_filter
# def my_filter(value):
#     return value.lower()

# Filter with jinja2 context

# @template_filter
# @contextfilter
# def my_filter(context, value):
#     print context
#     return value



import os
import time
import re
import traceback
import utilities as util
import itertools
from copy import deepcopy
from blinker import signal
import messages as messages


"""
---------------------------------------------------------------------------------------------------
Parser
---------------------------------------------------------------------------------------------------

Normalises data in a file and returns dicts ready to construct
page objects with

"""
class Parser(object):

    defaults = {
        "meta":{
            "title": None,
            "template": None,
            "description": None,
            "view": None,
            "output_file_extension": None
        }
    }

    def __init__(self, input_dir, root):
       self.input_dir = input_dir
       self.project_root = root

    def interpret(self, file_contents):
        return file_contents

    def attempt(self, file_contents, filepath):
        try:
            return self.interpret(file_contents)
        except:
            print messages.parser_decode_error % (filepath, self.accepts)
            traceback.print_exc()

    def read(self, file_path):
        source_file = open(file_path, 'r')
        file_contents = ""

        try:
            file_contents = source_file.read().decode('utf8')
        except:
            print messages.file_decode_error % (source_file)
            traceback.print_exc()
        return file_contents
    
    def parse(self, filepath, file_name, relative_path,  file_contents, mtime):
        page_data = {"data":[]}
        contents = self.attempt(file_contents, filepath)
        _defaults = deepcopy(self.defaults)
        page_data.update(_defaults)
        
        if contents != None: 
            if "meta" in contents:
                page_data["meta"].update(contents["meta"])
            if "data" in contents:
                page_data["data"] = contents["data"]

        page_data["meta"].update({
            "filename": file_name,
            "filepath": filepath,
            "relative_path": relative_path,
            "mtime": mtime
        })

        sig = signal("wranglerLoadItem")
        sig.send('item', item=file_name, file_contents=file_contents, mtime=mtime)

        return page_data

    def load(self, filepath):
        file_name, file_extension = os.path.splitext(filepath)
        file_path = os.path.join(self.project_root, filepath)
        relative_path = file_name.replace(self.input_dir+"/", "")     
        mtime = os.path.getmtime(filepath)
        file_contents = self.read(file_path)


        if file_extension:
            if file_extension.replace(os.extsep, "") in self.accepts:
                return self.parse(file_path, file_name, relative_path, file_contents, mtime)
            else:
                print messages.parser_error % (self.__class__.__name__, file_extension, self.accepts)




"""
---------------------------------------------------------------------------------------------------
NodeGraph
---------------------------------------------------------------------------------------------------

Allow for flat and tree navigation within a directory

"""

class NodeGraph():
    def __init__(self):
        self._nodes = {}
        self._root = None

    def add(self, item):
        self._nodes[item.path] = item

    def root(self, item):
        self._root = item

    def tree(self):
        return self._root

    def all(self):
        return self._nodes



"""
---------------------------------------------------------------------------------------------------
Node
---------------------------------------------------------------------------------------------------

Tree object that makes it easier to keep track of all the pages and how they relate to each other.

"""

class Node(object):
    def __init__(self, name, path, parent=None, cargo=None):
        self.name = name
        self.path = path
        self.cargo = cargo
        self.parent = parent
        self.children = list()
        self.is_index = False
        return None

    def add_child(self, node):
        self.children.append(node)

    def add_cargo(self, cargo=None):
        self.cargo = cargo

    def get_cargo(self):
        return self.cargo

    def get_children(self):
        return self.children

    def get_parent(self):
        return self.parent

    def get_siblings(self):
        siblings = []
        if self.parent != None:
            siblings = self.get_parent().get_children()
        return siblings

    def get_unique_siblings(self):
        siblings = []
        if self.parent != None:
            siblings = [sibling for sibling in  self.get_parent().get_children() if sibling.path != self.path]
        return siblings 

    def get_child(self, name):
        child = None

        for item in self.children:
            if name == "index" and item.is_index:
                child = item
            elif item.name == name:
                child = item  

        return child

    def get_child_pages(self):
        children = []

        directories = [d for d in self.get_parent().get_children() if d.tag == 'dir']
        
        for node in directories:
            index = node.get_child("index")

            if index:
                children.append(index)

        return children

    def get_parents(self):
        parents = []

        def _get(n):
            if n != None:
                parents.append(n.get_child("index"))
                _get(n.parent)

        _get(self.parent)

        return parents

    def get_parent_dirs(self):
        parents = []

        def _get(n):
            if n != None:
                parents.append(n)
                _get(n.parent)

        _get(self.parent)

        return parents
    

    def get_parents_siblings(self):
        _parent_siblings = []

        is_deep_enough = hasattr(self.get_parent(), "get_parent")

        if is_deep_enough:
            parent = self.get_parent().get_parent()

        if parent:
            directories = [d for d in parent.get_children() if d.tag == 'dir'] 

            for node in directories:
                index = node.get_child("index")

                if index:
                    _parent_siblings.append(index)

        return _parent_siblings


"""
---------------------------------------------------------------------------------------------------
Page
---------------------------------------------------------------------------------------------------

Provides methods for dealing with data loaded from a filesystem

"""
class Page(object):
    new_id = itertools.count().next
    def __init__(self, data, config):
        self.id = Page.new_id()
        self.data = data
        self.config = config
        self.set_name(self.data["meta"]["filepath"])
        self.data["meta"]["id"] = self.id

    def get_content(self):
        return self.data["data"]

    def get_metadata(self):
        return self.data["meta"]

    def get_file_ext(self):
        return self.data["meta"]["output_file_extension"] if "output_file_extension" in self.data["meta"] else None
    
    def set_name(self, filepath):
        self.name = os.path.splitext(filepath)[0]
        return self.name
    
    def get_name(self):
        return self.name

    def get_title(self):
        return self.data["meta"]["title"] if "title" in self.data["meta"] else None

    def get_short_title(self):
        return self.data["meta"]["alias"] if "alias" in self.data["meta"] else None

    def get_meta_description(self):
        return self.data["meta"]["description"] if "description" in self.data["meta"] else None

    def get_meta_keywords(self):
        return self.data["meta"]["keywords"] if "keywords" in self.data["meta"] else None

    def get_output_path(self):
        file_ext = self.get_file_ext()
        
        if file_ext == None:
            return self.output_path
        else:
            return self.output_path_no_ext + file_ext

    def get_relative_output_path(self):
        return self.relative_output_path

    def set_output_path(self, path):
        self.output_path = path[0]
        self.relative_output_path = path[1]
        self.output_path_no_ext = path[2]
        
        self._ext = self.output_path.split(".")[1]
        tidy_url = self.relative_output_path.replace("index.%s" % (self._ext), "")

        self.data["meta"]["segments"] = [segment for segment in self.relative_output_path.split("/")]
        self.data["meta"]["url"] = "/%s" % (tidy_url)
        self.data["meta"]["path"] = "/%s/" % (os.path.dirname(self.relative_output_path))


    def get_tidy_url(self):
        return self.data["meta"]["url"]

    def get_modified_time(self):
        return self.data["meta"]["mtime"]

    def get_file_path(self):
        return self.data["meta"]["filepath"]

    def get_tags(self):
        return self.data["meta"]["tags"] if "tags" in self.data["meta"] else None

    def get_related(self):
        return self.data["meta"]["related"] if "related" in self.data["meta"] else None

    def get_mtime(self):
        if "mtime" in self.data["meta"]:
            return self.data["meta"]["mtime"]
        return 0

    def relpath(self):
        return self.data["meta"]["relative_path"]

    def get_template(self):
        if "template" in self.data["meta"]:
            return self.data["meta"]['template']

        return self.config["default_template"]

    def before_render(self):
        return False;

    def on_save(self):
        return False;

    def show_in_navigation(self):
        return False if "hide_from_nav" in self.data["meta"] and self.data["meta"]["hide_from_nav"] else True

    def get_weight(self):
        return self.data["meta"]["weight"] if "weight" in self.data["meta"] else 0

    def get_thumbnail(self):
        return self.data["meta"]["thumbnail"] if "thumbnail" in self.data["meta"] else None

    def get_properties(self):
        return {
                "title": self.get_title(),
                "alias": self.get_short_title(),
                "description": self.get_meta_description(),
                "url": self.get_tidy_url(),
                "show_in_navigation": self.show_in_navigation(),
                "weight": self.get_weight(),
                "thumbnail": self.get_thumbnail()
        }

    def map_related_nodes(self, nodes, keyname):
        _data = []
        
        for node in nodes:
            if node:
                page = node.get_cargo()
                if page != None:
                    _data.append(page.get_properties())
        
        self.data["meta"][keyname] = _data

    def set_siblings(self, nodes):
        self.map_related_nodes(nodes, "siblings")

    def set_parents(self, nodes):
        self.map_related_nodes(nodes, "parents")

    def set_children(self, nodes):
        self.map_related_nodes(nodes, "children")

    def set_parents_siblings(self, nodes):
        self.map_related_nodes(nodes, "parents_siblings")

    def set_related_nodes(self, nodes):
        self.map_related_nodes(nodes, "related_nodes")

    def cleanup(self):
        self.data["meta"]["siblings"] = None
        self.data["meta"]["parents"] = None
        self.data["meta"]["parents_siblings"] = None
        self.data["meta"]["children"] = None
        self.data["meta"]["related_nodes"] = None


"""
---------------------------------------------------------------------------------------------------
Renderer
---------------------------------------------------------------------------------------------------

Generates templated content to pass to the writer

"""
class Renderer(object):
    def __init__(self, config, reporter, writer):
        self.config = config["wrangler"]
        self.reporter = reporter
        self.writer = writer
        self.init()
        return None

    def render(self, item):
        return str(item)

"""
---------------------------------------------------------------------------------------------------
Writer
---------------------------------------------------------------------------------------------------

Saves rendered items to the filesystem

"""
class Writer(object):
    def __init__(self, output_path, output_file_ext, reporter):
        self.output_path = output_path
        self.output_file_ext = output_file_ext
        self.reporter = reporter
        self.reporter.verbose("Ensure directory exists: \033[34m%s\033[0m" % (self.output_path))
        util.ensure_dir(self.output_path)
        return None

    def generate_output_path(self, filename):
        relative_output_path = util.clean_path(filename, self.output_file_ext)
        path = os.path.join(self.output_path, relative_output_path)
        path_no_ext = os.path.join(self.output_path, filename + os.extsep)
        return path, relative_output_path, path_no_ext

    def save(self, data):

        item=data[1]
        html=data[0]

        filename = item.get_output_path()
        new_directory = os.path.dirname(filename)

        if html == False:
            return False

        try: 
            sig = signal('wranglerBeforeSaveItem')
            sig.send('item', item=item, path=filename)
            util.ensure_dir(new_directory)
            file_object = open(filename, "w")
            file_object.write(html.encode('utf8'))
            self.reporter.print_stdout(item.get_file_path(), filename, item.get_template())
            item.on_save()
            siggy = signal('wranglerOnSaveItem')
            siggy.send('item', item=item, path=filename)

        except:
            print messages.file_write_error % (filename)
            traceback.print_exc()
            self.reporter.log_item_saved(item.get_file_path(), item.get_template(), 0)
            return False
        finally:
            self.reporter.log_item_saved(item.get_file_path(), item.get_template(), 1)
            return True




"""
---------------------------------------------------------------------------------------------------
Reporter
---------------------------------------------------------------------------------------------------
Shares our findings with the rest of the class

"""
class Reporter(object):
    def __init__(self, config, config_path):
        self.config = config["wrangler"]
        self.config_path = config_path
        self.log_data = []
        return None

    def print_stdout(self, original_path, new_path, template):
        self.verbose(messages.render_success % (original_path, new_path, template))


    def update_log(self, items, last_modified_time):
        file = open(self.config['compiled_templates_log'], 'w')
        print>>file, "this_render=%s, prev_render=%s" % (time.time(), last_modified_time)

        for item in items:
            print>>file, "%s, %s, %s" % (item[0], item[2], item[1])

        self.verbose(messages.write_log % (self.config['compiled_templates_log']))
    
    def log_item_saved(self, path, template, result):
        self.log_data.append([path, template, result])

    def get_last_build_time(self):
        """
        Get the last run time. Useful for checking what stuff's changed since we
        last ran the script.
        """
        last_time = 0
        if os.path.exists(self.config['build_cache_file']):
            with open(self.config['build_cache_file'], "r") as buildFile: 
                lines = map(float, buildFile.readline().split())
                if len(lines) > 0:
                    last_time = lines[0]
        return last_time

    def get_config_modified_time(self):
        """
        Force a re-render if the config file has changed (global paths may be invalid, etc..)
        """
        mtime = 0
        if os.path.exists(self.config_path):
            mtime = os.path.getmtime(self.config_path)
        return mtime

    def set_last_build_time(self, update_time=None):
        """
        Save the last successful run time in a var
        """
        if (update_time == None):
            update_time = time.time()

        self.update_log(self.log_data, self.get_last_build_time())

        with open(self.config['build_cache_file'], "w") as buildFile:
            buildFile.write("%s" % (update_time))

    def pretty(self, colour, message):
        colours = {
            "green": "\033[32m",
            "red": "\033[31m",
            "blue": "\033[34m",
            "basic": "\033[0m"
        }
        
        return "%s%s%s" % (colours[colour], message, colours["basic"]) 

    def log(self, message, status=None):
        msg = message
        if status:
            msg = self.pretty(status, message)
        print msg

    def verbose(self, message, status=None):
        if self.config["verbose"] == True:
            self.log(message, status)


"""
---------------------------------------------------------------------------------------------------
Decoration
---------------------------------------------------------------------------------------------------
"""

def template_filter(inner):
    jinja_filter = signal('template_filter')

    @jinja_filter.connect
    def sig(sender, **kwargs):
        return inner

    return sig

def extension(inner):
    wranglerExtension = signal('wranglerExtension')

    @wranglerExtension.connect
    def sig(sender, **kwargs):
        return inner

    return sig

def before_render(inner):
    directive = signal('wranglerBeforeRender')

    @directive.connect
    def sig(sender, **kwargs):
        return inner(**kwargs)

    return sig

def after_render(inner):
    directive = signal('wranglerAfterRender')

    @directive.connect
    def sig(sender, **kwargs):
        return inner(**kwargs)

    return sig

def load_item(inner):
    directive = signal('wranglerLoadItem')

    @directive.connect
    def sig(sender, **kwargs):
        return inner(**kwargs)

    return sig

def render_item(inner):
    directive = signal('wranglerRenderItem')

    @directive.connect
    def sig(sender, **kwargs):
        return inner(**kwargs)

    return sig

def save_item(inner):
    directive = signal('wranglerBeforeSaveItem')

    @directive.connect
    def sig(sender, **kwargs):
        return inner(**kwargs)

    return sig



